function [Wq, W, Q, rho, equilibrium_prob] = GM1_queue(Alpha, Beta, Nu, Tau, mu, burnin, data, N, x1, x2)
%
% Given a set of single parameters (alpha,beta,nu,tau) (or a set of vectors,
% for example the output of the Gibbs sampler)
% this program computes the (predictive) queue and system waiting time
% distributions (Wq and W), as well as the (predictive) system size distribution (Q) of a
% dPlN/M/1 queueing system. 
%
%
% INPUT Alpha,Beta,Nu,Tau = sample of values (maybe from the posterior
%                           distribution) of the dPlN parameters.
%       mu = service rate (expected service time = 1/mu).
%       burnin = the predictive quantities are computed from iteration
%                burnin+1; if a set of single parameters is the input, fix
%                burnin = 0;
%       data = data generated from a dPlN; 
%       N = number of Tam samples (default = 100) to be used in
%           TAMsamples.m
%       x1 = points to evaluate W and Wq
%       x2 = points to evaluate Q
%
% OUTPUT t = 
%       Wq = (predictive) queue Waiting time distribution;
%       W = (predictive) system Waiting time distribution;
%       Q = (predictive) system size distribution;
%       equilibrium_prob = predictive equilibrium probability
%                         (as equation 8 of Ramirez et al.)
%       rho = (posterior estimate of) the traffic intensity.  
%
% EXAMPLE 1: Given a set of single paramters (alpha,beta,nu,tau) compute the
%            quantities of interest of the dPlN/M/1 for a given service rate mu:
%
%           alpha = 2; beta = .5; nu = 0; tau = 1; [y] = simulation(alpha,beta,nu,tau,1000);
%           x1 = 0:0.001:1; x2 = 0:7; N = 100; data=exp(y); burnin = 0; mu = 10;
%           [Wq, W, Q, rho, equilibrium_prob] = GM1_queue(alpha, beta, nu, tau, mu, burnin, data, N, x1, x2);
%           traffic_intensity = rho
%           plot(x1,Wq); figure; plot(x1,W); figure;
%           plot(x2,Q,'.')
%
% EXAMPLE 2: Given the output of the Gibbs sampler, compute the
%            predictive quantities of interest of the dPlN/M/1 for a given service rate mu:
%
%           [y] = simulation(2,.5,1,1,1000);
%           [Alpha, Beta, Nu, Tau] = Gibbs_dPlN(y,[1 1 0 4 .5 .5],10000);
%           x1 = 0:0.001:1; x2 = 0:7; N = 100; data=exp(y); burnin = 8000; mu = 10;
%           [Wq, W, Q, rho, equilibrium_prob] = GM1_queue(alpha, beta, nu, tau, mu, burnin, data, N, x1, x2);
%           traffic_intensity = rho
%           posterior_probability_of_equilibrium = equilibrium_prob
%           plot(x1,Wq); figure; plot(x1,W); figure;
%           plot(x2,Q,'.')

iters = length(Alpha);
W = zeros(1,length(x1));
Wq = zeros(1,length(x1));
Q = zeros(1, length(x2));
cont = 0;
Rho = [];

for i = (burnin+1):iters
    alpha = Alpha(i);
    beta = Beta(i);
    nu = Nu(i);
    tau = Tau(i);
    rho = ( (alpha -1)*(beta +1) )/(mu*alpha * beta* exp(nu + (tau^2)/2));
    Rho = [Rho rho];
    if (alpha <=1) | (rho<1)
        [r0] = root(data, mu, alpha, beta, nu, tau, N);
        Wq = 1-r0*exp(-mu*(1-r0).*x1) + Wq;
        W = 1-exp(-mu*(1-r0).*x1) + W;
        Q = (1-r0)* (r0.^x2) + Q;
        cont = cont + 1; 
    end
end

% Predictive quantities
W = W / cont; 
Wq = Wq / cont; 
Q = Q / cont; 
equilibrium_prob = cont/(iters-burnin);
predictive_rho = mean(Rho);


