function [Alpha, Beta, Nu, Tau] = Gibbs_dPlN(y, prior_par, iters)

% Gibbs algorithm to estimate the dPlN distribution.
%
% Algorithm from Ramirez et al. 
% INPUT y = data generated from a Normal Laplace distribution (X = exp(Y)
%           is the dPlN)
%       prior_par = vector of parameters for the prior distributions,
%              prior_par = [a b m k c d], (a,b) for the inverse gamma prior,
%              (m,k) for nu|tau, and (c,d) for the gamma priors of alpha and beta.
%       iters = total number of iterations.
%
% OUTPUT Alpha = sampled value of alpha at each iteration
%        Beta = sampled value of beta at each iteration
%        Nu =  sampled value of nu at each iteration
%        Tau = sampled value of tau at each iteration
%
%
% EXAMPLE 1: [y] = simulation(.25,.5,0,1,1000);
%            [Alpha, Beta, Nu, Tau] = Gibbs_dPlN(y,[1 1 0 4 .5 .5],100000)
%
% EXAMPLE 2: load real_data; y = log(aon);
%            [Alpha, Beta, Nu, Tau] = Gibbs_dPlN(y,[1 1 0 4 .5 .5],100000)
%


Alpha = zeros(1,iters);
Beta = zeros(1,iters);
Nu = zeros(1,iters);
Tau = zeros(1,iters);
a = prior_par(1);
b = prior_par(2);
m = prior_par(3);
k = prior_par(4);
c = prior_par(5);
d = prior_par(6);
n=length(y);

%
% Starting values: MLE's (Reed and Jorgensen, 2004)
z = fminsearch(@myfun1,[1,1,1],[],y);
tau = z(3);
alpha= z(1);
beta= z(2);
nu = mean(y) - 1/alpha + 1/beta;

Alpha(1) = alpha;
Beta(1) = beta;
Nu(1) = nu;
Tau(1) = tau; 

%
% Gibbs sampler
for i = 2:iters
i
%
% Generation of the values of W and Z
%
    prob = normcdf(-(y-nu+beta*tau*tau)/tau)./(normcdf(-(y-nu+beta*tau*tau)/tau)+...
        (1-normcdf(-(y-nu-alpha*tau*tau)/tau)).*exp((0.5/(tau^2))*((y-nu-alpha*tau*tau).^2-...
        (y-nu+beta*tau*tau).^2)));
    u = rand(1,n);
    u1 = find(u<prob);
    w(u1) = -truncnormvect(-(y(u1)-nu+beta*(tau^2)),tau,0);
    u2 = find(u>=prob);
    w(u2) = truncnormvect((y(u2)-nu-alpha*(tau^2)),tau,0);   
    z = y-w;

    
%
% Generation of E1 and E2
%   
    e1 = exprnd(1/(alpha+beta),1,length(w));
    e1(w>0) = e1(w>0) + w(w>0);
    e2 = e1-w;
%
% Generate nu and tau
% 
    [nu,tau]=nutau(z,m,k,a,b);
%
% Generate alpha and beta
% 
    alpha = gamrnd(c+n,1/(d+n*mean(e1)));
    beta = gamrnd(c+n,1/(d+n*mean(e2)));
%
%

    Alpha(i) = alpha;
    Beta(i) = beta;
    Nu(i) = nu;
    Tau(i) = tau;   
end    




