function [t, Wq, rho, equilibrium_prob] = MG1_queue(Alpha, Beta, Nu, Tau, lambda, burnin, data, N, T)
%
% Given a set of single parameters (alpha,beta,nu,tau) (or a set of vectors,
% for example the output of the Gibbs sampler)
% this program computes the (predictive) queue waiting time distribution (Wq),
% of a M/dPlN/1 queueing system. Note that alpha must be larger that 1, if not
% the queue is not stable and Wq does not exist.
%
% INPUT Alpha,Beta,Nu,Tau = sample of values (maybe from the posterior
%                           distribution) of the dPlN parameters.
%       Lambda = Interarrival rate (expectal interarrival time = 1/lambda).
%       burnin = the predictive quantities are computed from iteration
%                burnin+1; if a set of single parameters is the input, fix
%                burnin = 0;
%       data = data generated from a dPlN; 
%       N = number of Tam samples (default = 100) to be used in
%           TAMsamples.m
%       T = small time increment (recursion method for inverting the LT).
%
%
% OUTPUT t = times where the predictive Waiting time distribution is
%            evaluated.
%       Wq = (predictive) Waiting time distribution;
%       predictive_rho = (posterior estimate of) the traffic intensity.  
%       equilibrium_prob = predictive equilibrium probability
%                         (as equation 8 of Ramirez et al.)
% EXAMPLE 1: Given a set of single paramters (alpha,beta,nu,tau) compute the
%            quantities of interest of the M/dPlN/1 for a given arrival rate lambda:
%
%           alpha = 2; beta = .5; nu = 0; tau = 1; [y] = simulation(alpha,beta,nu,tau,1000);
%           lambda = 0.1; burnin = 0; data = exp(y); N = 100; T = 0.01;
%           [t, Wq, rho, equilibrium_prob] = MG1_queue(alpha, beta, nu, tau, lambda, burnin, data, N, T);
%           traffic_intensity = rho
%           plot(t,Wq)
%
% EXAMPLE 2: Given the output of the Gibbs sampler, compute the
%            predictive quantities of interest of the dPlN/M/1 for a given service rate mu:
%
%           [y] = simulation(2,.5,1,1,1000);
%           [Alpha, Beta, Nu, Tau] = Gibbs_dPlN(y,[1 1 0 4 .5 .5],10000);
%           [t, Wq, rho, equilibrium_prob] = MG1_queue(Alpha, Beta, Nu, Tau, 0.1, 8000, exp(y), 100, 0.1)
%           predictive_traffic_intensity = rho
%           posterior_probability_of_equilibrium = equilibrium_prob
%           plot(t,Wq)


iters = length(Alpha);
Wtime = zeros(1,10000);
count = 0;
Rho = [];
x = [];

for i = (burnin+1):iters
    alpha = Alpha(i);
    beta = Beta(i);
    nu = Nu(i);
    tau = Tau(i);
    if alpha <= 1
        rho = Inf; % the system is not stable
    else
        expected_service = alpha*beta*exp(nu + tau^2 /2)/((alpha-1)*(beta +1));
        rho = lambda*expected_service; 
                if rho<1 % the system is stable
                    [t,w] = TAMsamples(data, alpha, beta, nu, tau, N);
                    [r,n] = recursion_method(t,w,lambda,rho,T);
                    Wtime(1:length(r)) = Wtime(1:length(r)) + r;
                    x = [x n];
                    count = count + 1;
                end
                Rho = [Rho rho];
    end
end

% Predictive quantities
nmin = round(min(x)/2);
Wtime = Wtime(1:nmin); 
t = [0 T*(1:(nmin-1))];
Wq = Wtime / count; 
equilibrium_prob = count/(iters-burnin);
rho = mean(Rho);



            