function [t,w] = TAMsamples(data, alpha, beta, nu, tau, N)
%
% Computes the TAM samples and weights to approximate the Laplace Transform
% of the DPLN distribution, using the algorithm described in Ramirez et al.
% (Section 3).
%
% INPUT data = data generated from a dPlN distribution
%       alpha,beta,nu,tau = parameters of the model
%       N = number of TAM samples. Default N = 100
%
% OUTPUT t = TAM samples (quantiles), w = weights.
%
%
%

% Computation of the value X0 such that the UTAM is applied to the left of
% X0 and the GTAM to the right of X0.

if alpha>1 %if the expected value of the distribution exists
    EX = alpha*beta*exp(nu + tau^2 /2)/((alpha-1)*(beta + 1));
else %we compute the median (numerically)
    range = -100:.001:100;
    [G] = cdf_NL(alpha,beta,nu,tau,range);
    i = find (G<.5);
    u = range(i(length(i)));
    EX = exp(u);
end
[X0] = cdf_NL(alpha,beta,nu,tau,log(EX));

% Pick a set of N probabilities p_i:
r = [.2 .3 .4 .5 .6 .7 .8 .9];% to choose the optimum percentage of probabilities
q = .1:.05:.9; % geometric parameter
Q = length(q)*length(r);
means = zeros(1,Q); 
T = zeros(Q,N); % records the tam samples for every value of the grid
W = zeros(Q,N);
cont = 1;

data = sort(data);
L = length(data);
Fn = (0:(L-1))/L; %empirical cdf of the data


for k = 1:length(r)
    for l = 1:length(q)
        a = X0/ (round((1-r(k) )*N) );
        p1 = a:a:X0; % UTAM
        ss = 1;
        long = 0;
        bound = round(r(k)*N);
        while long < bound % GTAM
            p2 = 1-q(l).^((1:bound*2)/ss);
            [i] = find(p2>X0);
            p2 = p2(i);
            [i] = find(p2<=.99999999999999);
            p2 = [p2(i)];
            long = length(p2);
            ss = ss+1;
        end 
        p2 = p2(1:bound);
        p = [p1 p2]; %set of N probabilities

        % Computation of the TAM samples, t.
        
        % Starting value x0 of the Newton Raphson algorithm
        for i = 1:length(p)
            [j] = find(Fn<p(i)+.01 & Fn>p(i)-.01);
            x0(i) = data(j(1));
        end
        % Tam samples: t.
        [T(cont,:)] = newton_raphson(x0,p,alpha,beta,nu,tau);
        
        % Weights.
        a = (p(1) +p(2))/2;
        b = 1- (p(N-1)+p(N))/2;
        w = (p(3:N)-p(1:N-2))/2;
        w = [a w b];
        W(cont,:) = w;
        if alpha>1
            means(cont) = sum(W(cont,:).*T(cont,:)); % tam mean
        else
            u = find(cumsum(w)<.5);
            i = u(length(u));
            means(cont) = T(cont,i); % tam median
        end
        cont = cont+1;
    end
end

% Choose the optimal
error = abs(means-EX);
[m,j] = min(error);
t = T(j,:);
w = W(j,:);
clear T W