function [NLpdf, alpha, beta, nu, tau] = predictive(y, Alpha, Beta, Nu, Tau, burnin)

% Predictive quantities given the output of the Gibbs sampler.
% 
% INPUT y = data generated from a Normal Laplace distribution.
%       Alpha, Beta, Nu, Tau = output of the Gibbs sampler.
%       burnin = the predictive quantities are computed from the burnin+1
%       iteration.
%
% OUTPUT NLpdf = predictive Normal Laplace density
%        alpha, beta, nu, tau = predictive estimates of the parameter of
%        the model.
%
% EXAMPLE: [y] = simulation(.25,.5,1,1,5000);
%          [Alpha, Beta, Nu, Tau] = estimate_dPlN(y,[1 1 0 4 .5 .5],20000,80000)
%          [NLpdf, alpha, beta, nu, tau] = predictive(Alpha, Beta, Nu, Tau, burnin)
%
%
%
iters = length(Alpha);

alpha = mean(Alpha(burnin+1:iters));
beta = mean(Beta(burnin+1:iters));
nu = mean(Nu(burnin+1:iters));
tau = mean(Tau(burnin+1:iters));
NLpdf= zeros(1,length(y));

y = sort(y); %to plot later easily
for i = burnin+1:iters
    [g] = pdf_NL(Alpha(i),Beta(i),Nu(i),Tau(i),y);
    NLpdf = NLpdf + g;
end

NLpdf = NLpdf/(iters-burnin); % posterior predictive density.

% hist1(y,20); hold on; plot(y,NLpdf);
