function [r,n] = recursion_method(t,w,lambda,rho,T)
%
% Recursive method to invert the LT of the equilibrium waiting time
% distribution in a M/dPlN/1 queueing system. See Shortle et al. (2003)
%
% INPUT t,w = tam samples and corresponding weights obtained from
%             TAMsamples.m
%       lambda = interarrival rate
%       rho = traffic intensity
%       T = small time increment
%
% OUTPUT r = vector such that r(i) = P(Wq <= iT)
%        n+1 = length of r;
%
% EXAMPLE: alpha = 2; beta = .5; nu = 0; tau = 1;
%          [y] = simulation(alpha,beta,nu,tau,1000);
%          data = exp(y); N = 100; lambda = 0.5; T = 0.01;
%          rho = lambda*alpha*beta*exp(nu + tau^2 /2)/((alpha-1)*(beta +1));
%          if rho<1
%               [t,w] = TAMsamples(data, alpha, beta, nu, tau, N);     
%               [r,n] = recursion_method(t,w,lambda,rho,T);
%               plot([0 T*(1:n)],r);
%          else
%               disp('try another lambda s.t. rho<1')
%          end

R = round(t/T);
n = max(R);
[cont] = find(R == 0);
c0 = sum(w(cont));
r0 = (1-rho)/(1+lambda*T*(c0-1));
c = zeros(1,n);
for i = 1:n
    [cont] = find(R == i);
    c(i) = sum(w(cont));
end
v = n:-1:1;
cinvert = c(v); % c_n, c_n-1,...,[c_i,..., c_2, c_1].
cons = 1/(1+lambda*T*(c0-1));
r = [cons*(r0-lambda*T*c(1)*r0)];
for i = 2:n
    a = r(1:i-1);
    b = cinvert( (n-i+2):n );
    r(i) = cons*(r(i-1)-lambda * T * (c(i)*r0 + sum (a.*b)));
end
r = [r0 r];